import { ObjectLiteral } from "../../common/ObjectLiteral";
import { EntityMetadata } from "../../metadata/EntityMetadata";
import { ColumnMetadata } from "../../metadata/ColumnMetadata";
import { RelationMetadata } from "../../metadata/RelationMetadata";
export interface JunctionInsert {
    relation: RelationMetadata;
    junctionEntities: ObjectLiteral[];
}
export interface JunctionRemove {
    relation: RelationMetadata;
    junctionRelationIds: any[];
}
export interface RelationUpdate {
    relation: RelationMetadata;
    value: any;
}
/**
 */
export declare class Subject {
    metadata: EntityMetadata;
    entity: ObjectLiteral;
    _databaseEntity?: ObjectLiteral;
    canBeInserted: boolean;
    canBeUpdated: boolean;
    mustBeRemoved: boolean;
    /**
     * List of relations which need to be unset.
     * This is used to update relation from inverse side.
     */
    relationUpdates: RelationUpdate[];
    diffColumns: ColumnMetadata[];
    diffRelations: RelationMetadata[];
    /**
     * When subject is newly persisted it may have a generated entity id.
     * In this case it should be written here.
     *
     * @deprecated use newlyGeneratedId instead. Difference between this and newly generated id
     * is that newly generated id hold value itself, without being in object.
     * When we have generated value we always have only one primary key thous we dont need object
     */
    entityId: any;
    /**
     * When subject is newly persisted it may have a generated entity id.
     * In this case it should be written here.
     *
     */
    newlyGeneratedId: any;
    /**
     * Used in newly persisted entities which are tree tables.
     */
    treeLevel: number;
    /**
     * Date when this entity is persisted.
     */
    date: Date;
    junctionInserts: JunctionInsert[];
    junctionRemoves: JunctionRemove[];
    constructor(metadata: EntityMetadata, entity?: ObjectLiteral, databaseEntity?: ObjectLiteral);
    readonly entityTarget: Function | string;
    /**
     * Returns readable / loggable name of the entity target.
     */
    readonly entityTargetName: string;
    readonly id: ObjectLiteral | undefined;
    readonly mixedId: any;
    readonly mustBeInserted: boolean;
    readonly mustBeUpdated: boolean;
    readonly hasRelationUpdates: boolean;
    databaseEntity: ObjectLiteral | undefined;
    compareId(id: ObjectLiteral): boolean;
    /**
     * Differentiate columns from the updated entity and entity stored in the database.
     */
    private buildDiffColumns();
    /**
     * Difference columns of the owning one-to-one and many-to-one columns.
     */
    private buildDiffRelationalColumns();
    /**
     * Gets id of the persisted entity.
     * If entity itself has an id then it simply returns it.
     * If entity does not have an id then it returns newly generated id.
     */
    getPersistedEntityIdMap(): any;
}
