"use strict";
const Subject_1 = require("./Subject");
const SubjectGroup_1 = require("./SubjectGroup");
/**
 */
class SubjectCollection extends Array {
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Pushes subject to the collection only in the case if subject with same entity target and entity id
     * does not exist in the collection.
     */
    pushIfNotExist(subject) {
        const existSubject = this.findByEntityLike(subject);
        if (!existSubject)
            this.push(subject);
    }
    /**
     * Finds subject with a given entity.
     */
    findByEntity(entity) {
        return this.find(subject => subject.entity === entity);
    }
    /**
     * Finds subject where entity like given entity or subject's entity.
     * Comparision made by entity id.
     */
    findByEntityLike(entityTargetOrSubject, maybeEntity) {
        const entityTarget = entityTargetOrSubject instanceof Subject_1.Subject ? entityTargetOrSubject.metadata.target : entityTargetOrSubject;
        const entity = entityTargetOrSubject instanceof Subject_1.Subject ? entityTargetOrSubject.entity : maybeEntity;
        return this.find(subject => {
            return subject.entityTarget === entityTarget && subject.metadata.compareEntities(subject.entity, entity);
        });
    }
    /**
     * Groups given Subject objects into groups separated by entity targets.
     */
    groupByEntityTargets() {
        return this.reduce((groups, operatedEntity) => {
            let group = groups.find(group => group.target === operatedEntity.entityTarget);
            if (!group) {
                group = new SubjectGroup_1.SubjectGroup(operatedEntity.entityTarget);
                groups.push(group);
            }
            group.subjects.push(operatedEntity);
            return groups;
        }, []);
    }
    findByEntityId(entityTarget, id) {
        return this.find(subject => {
            return subject.entityTarget === entityTarget && subject.metadata.compareEntityMixedIds(subject.mixedId, id);
        });
    }
    findByDatabaseEntityLike(entityTarget, entity) {
        return this.find(subject => {
            return subject.entityTarget === entityTarget && subject.metadata.compareEntities(subject.databaseEntity, entity);
        });
    }
    findByDatabaseId(entityTarget, id) {
        return this.find(subject => {
            const databaseEntityMixedId = subject.metadata.getEntityIdMixedMap(subject.databaseEntity);
            return subject.entityTarget === entityTarget && subject.metadata.compareEntityMixedIds(databaseEntityMixedId, id);
        });
    }
    hasWithEntity(entity) {
        return !!this.findByEntity(entity);
    }
}
exports.SubjectCollection = SubjectCollection;

//# sourceMappingURL=SubjectCollection.js.map
